Animation = {}

function Animation:new()
  self.__index = self
  local a = {}
  
  a.Sprite = {}
  a.CurrentAnimation = 1
  
  function a:getDirection()
    local currentAnim = self.CurrentAnimation
    local anim = self.Sprite[currentAnim]
    return anim.ScaleX
  end
  
  --TODO: rce - Create and offset based on scale
  function a:addAnimation(path, rows, columns, duration, offset, startGid, endGid, name)
    local index = name or (#self.Sprite + 1)
    local sprite = self:newAnimation(love.graphics.newImage(path), rows, columns, duration, offset, startGid, endGid)
    self.Sprite[index] = sprite
  end
  
  function a:moveTo(id)
    self.CurrentAnimation = id
  end
  
  function a:resetAnimations()
    self.CurrentAnimation = 1
    for k, sprite in ipairs(self.Sprite) do
      sprite.CurrentTime = 0
    end
  end
  
  function a:nextAnimation()
    self.CurrentAnimation = self.CurrentAnimation + 1
    if self.CurrentAnimation > #self.Sprite then
      self.CurrentAnimation = 1
      return self.CurrentAnimation, true
    end
    return self.CurrentAnimation, false
  end
  
  function a:getDuration()
    local currentAnim = self.CurrentAnimation
    local anim = self.Sprite[currentAnim]
    return anim.Duration
  end
  
  function a:update(dt)
    local currentAnim = self.CurrentAnimation
    local anim = self.Sprite[currentAnim]
    if anim ~= nil then
      anim.CurrentTime = anim.CurrentTime + dt
      if anim.CurrentTime >= anim.Duration then
        anim.CurrentTime = anim.CurrentTime - anim.Duration
        return true, 0
      end  
      return false, anim.CurrentTime
    end
    return false, nil
  end
  
  function a:draw(bounds, scaleX, scaleY)
    local currentAnim = self.CurrentAnimation
    local anim = self.Sprite[currentAnim]
    local sprite = anim.spritesheet
    local quads = anim.quads
    
    local sx = scaleX
    local sy = scaleY
    
    local x = bounds.X + (anim.ScaledWidth/2) - anim.xOffset
    local y = bounds.Y + anim.yOffset
    
    if anim.Duration > 0 then
      local spriteNum = math.ceil(anim.CurrentTime / anim.Duration * #quads)
      if spriteNum == 0 then spriteNum = 1 end
      love.graphics.draw(sprite, quads[spriteNum], x, y, 0, sx, sy, anim.HalfWidth, 0)
    else
      love.graphics.draw(sprite, x, y, 0, sx, sy, anim.HalfWidth, 0)
    end
  end
  
  return setmetatable(a, self)
end

function Animation:newAnimation(image, rows, columns, duration, offset, startGid, endGid)
    local animation = {}
    animation.spritesheet = image
    animation.quads = {}
    
    local height = image:getHeight() / rows
    local width = image:getWidth() / columns
    local sgid = startGid or 0
    local egid = endGid or (rows * columns)
    local gid = 0
 
    for y = 0, image:getHeight() - height, height do
        for x = 0, image:getWidth() - width, width do
            if gid >= sgid and gid <= egid then
              table.insert(animation.quads, love.graphics.newQuad(x, y, width, height, image:getDimensions()))
            end
            gid = gid + 1
        end
    end
    
    animation.ScaleX = SCALE
    animation.ScaleY = SCALE
    animation.ScaledWidth = width * SCALE
    animation.ScaledHeight = height * SCALE
    animation.Width = width
    animation.Height = height
    animation.HalfWidth = width / 2
    animation.HalfHeight = height / 2
    animation.Duration = duration or 1
    animation.CurrentTime = 0
    
    if offset ~= nil then
      animation.xOffset = offset.X or 0
      animation.yOffset = offset.Y or 0
    else 
      animation.xOffset = 0
      animation.yOffset = 0
    end
    
    return animation
end

function newAnimationFromXML(anim, img_xml)
  local source = img_xml.spritesheet
  local name = img_xml.name
  local rows = tonumber(img_xml.rows)
  local cols = tonumber(img_xml.columns)
  local duration = tonumber(img_xml.duration)
  local offsetX = tonumber(img_xml.offsetx)
  local offsetY = tonumber(img_xml.offsety)
  local startgid = tonumber(img_xml.startgid)
  local endgid = tonumber(img_xml.endgid)
  
  anim:addAnimation(source, rows, cols, duration, Vector:new(offsetX, offsetY), startgid, endgid, name)
end