require('systems.system')
require('components.componentgenerator')

CameraSystem = System:new()

local function getCenter(camera)
  local x = (love.graphics.getWidth() * (1/camera.Scale))/2
  local y = (love.graphics.getHeight() * (1/camera.Scale))/2
  return Vector:new(x,y)
end

function CameraSystem:load(systemManager)
  local camera = ComponentGenerator:generate('Camera')
  self:addComponent(camera)
  
  self.ZoomScale = .25
  self.MinZoom = 1
  self.MaxZoom = 5
  
  self.SystemManager = systemManager
end

function CameraSystem:set()
  local camera = self:findComponent('Camera')
  local w,h = love.graphics.getWidth(), love.graphics.getHeight()
  local dx, dy = w/2 * (camera.Scale-1), h/2 * (camera.Scale-1)
  
  love.graphics.print("Camera X: "..camera.Pos.X.." Y: "..camera.Pos.Y, 10, 25)  
  love.graphics.print("Camera Scale: "..camera.Scale, 10, 40)
  
  love.graphics.setScissor(0, 0, w, h)
  
  --Camera draw logic
  love.graphics.push()
  love.graphics.translate(camera.Offset.X, camera.Offset.Y)
  love.graphics.scale(1 / camera.Scale)
  love.graphics.rotate(-camera.Rotation)
  love.graphics.translate(-camera.Pos.X+dx, -camera.Pos.Y+dy)
  
  --Box of zoomed viewport
  love.graphics.rectangle("line", camera.Pos.X, camera.Pos.Y, w, h)
end

function CameraSystem:unset()
  love.graphics.pop()
  love.graphics.setScissor()
end

local function zoomCamera(system, scrolled, camera, pos)
  if scrolled ~= nil then
    local p_center = getCenter(camera)
    local w,h = love.graphics.getWidth(), love.graphics.getHeight()
    
    camera.Scale = camera.Scale - (scrolled.Y * system.ZoomScale)
    
    if (camera.Scale < system.MinZoom) then 
      camera.Scale = system.MinZoom
    elseif (camera.Scale > system.MaxZoom) then 
      camera.Scale = system.MaxZoom 
    end
  else
    camera.Offset = Vector:new(0,0)
  end
end

function CameraSystem:update(dt)
  local camera = self:findComponent('Camera')
  local dragged,clicked,scrolled,pos = self.SystemManager:findSystem('Input'):poll('Mouse', 'Dragged', 'Clicked', 'Scrolled', 'Position')
  
  if clicked ~= nil then
    
  end
  
  zoomCamera(self, scrolled, camera, pos)
  
  if dragged ~= nil then
    camera.Pos.X = camera.Pos.X + (dragged.X * camera.Scale)
    camera.Pos.Y = camera.Pos.Y + (dragged.Y * camera.Scale)
  end
end